"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Add markup to `text`, at the places indicated by `boundaries`.
 *
 * We're doing this with exact word boundaries and `String#slice`,
 * because if you were to do this with a regular expression and
 * `String#replace`, you could match against the start and end tags.
 *
 * So for example, this could happen:
 *
 *   markup('test abc test', ['abc', 'mark'], '<mark>', '</mark>');
 *   #=> 'test <<mark>mark</mark>>abc</<mark>mark</mark>> test'
 *
 * ...yeah, oops.
 */
exports.default = (function (text, boundaries, startTag, endTag) {
    if (startTag === void 0) { startTag = ''; }
    if (endTag === void 0) { endTag = ''; }
    // Keep track of successful 'markups', instead of the using the index of the
    // reduce iterations, since the reducer will bail out if the boundaries are
    // incorrect or unworkable.
    var markups = 0;
    var tagLength = startTag.length + endTag.length;
    return boundaries.reduce(function (acc, next) {
        if (!Array.isArray(next)) {
            throw new TypeError('Incorrect boundary. `boundaries` must be Array<number, number>');
        }
        if (next[1] < next[0]) {
            throw new Error('Incorrect boundary. boundary[1] must be greater than boundary[0].');
        }
        // The only time we don't need to account for tag length is when we're
        // marking up the first matched word.
        var first = markups === 0;
        // Don't do anything if the boundaries are outside the length of the `text`.
        if (next[1] < 0 || next[0] > text.length) {
            return acc;
        }
        var offset = first ? 0 : tagLength * markups;
        var startIndex = first ? next[0] : next[0] + offset;
        var endIndex = first ? next[1] : next[1] + offset;
        // Assemble the pieces.
        var start = acc.slice(0, startIndex);
        var word = acc.slice(startIndex, endIndex);
        var end = acc.slice(endIndex);
        markups += 1;
        return "" + start + startTag + word + endTag + end;
    }, text);
});
